# Name: libsetuproi/filechoice.py
# Purpose: wxPython widget for selecting a file from a list
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006-2009 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.


import os
import os.path
import wx

__all__ = ['FileChoice', 'EVT_FILE_CHOICE']


#
# wxPython event generated by the FileChoice widget
#

EVT_FILE_CHOICE_ID = wx.NewId()


def EVT_FILE_CHOICE(win, id, func):
    """
    Register to receieve selection events from a FileChoice.
    """
    win.Connect(id, -1, EVT_FILE_CHOICE_ID, func)


class FileChoiceEvent(wx.PyCommandEvent):
    """
    wxPython event emitted when the selection of a FileChoice is changed.
    """

    def __init__(self, id, fileName):
        wx.PyCommandEvent.__init__(self, EVT_FILE_CHOICE_ID, id)
        self.fileName = fileName

    def Clone(self):
        return FileChoiceEvent(self.GetId(), self.fileName)


class FileChoice(wx.Panel):
    NO_SELECTION = 'Don\'t plot'

    def __init__ (self, parent, id, choices=[], buttonLabel='Browse',
     dialogTitle='File Selection', defaultDirectory=None,
     fileMask='All files (*.*)|*', shareDirectory=None, **kwds):

        wx.Panel.__init__ (self, parent, id, **kwds)
        self.dialogTitle = dialogTitle
        if defaultDirectory is None:
            defaultDirectory = os.getcwd()
        self.defaultDirectory = defaultDirectory
        self.shareDirectory = shareDirectory
        self.fileMask = fileMask
        self.fileNames = [os.path.split(x) for x in choices]

        self.chooser = wx.Choice(self, -1, choices=[self.NO_SELECTION])
        self.browser = wx.Button(self, -1, buttonLabel)

        szr = wx.BoxSizer(wx.HORIZONTAL)
        szr.Add(self.chooser, 1, wx.RIGHT|wx.ALIGN_CENTER_VERTICAL, 1)
        szr.Add(self.browser, 0, wx.RIGHT|wx.ALIGN_CENTER_VERTICAL, 1)
        self.SetSizer(szr)
        self.Fit()

        wx.EVT_CHOICE(self, self.chooser.GetId(), self._on_choose)
        wx.EVT_BUTTON(self, self.browser.GetId(), self._on_browse)

        self._update_chooser()
        self.chooser.SetSelection(0)

    def _on_choose(self, evt):
        self._select_file(self.chooser.GetSelection())

    def _on_browse(self, evt):
        if self.shareDirectory is not None:
            defaultDirectory = self.shareDirectory.defaultDirectory
        else:
            defaultDirectory = self.defaultDirectory

        dlg = wx.FileDialog(self, style=wx.OPEN|wx.MULTIPLE,
            message=self.dialogTitle, wildcard=self.fileMask,
            defaultDir=defaultDirectory)

        if dlg.ShowModal() == wx.ID_OK:
            directory = dlg.GetDirectory()
            fileNames = [x for x in dlg.GetFilenames()
                            if (directory, x) not in self.fileNames]

            if self.shareDirectory is not None:
                self.shareDirectory.defaultDirectory = directory
            else:
                self.defaultDirectory = directory

            if fileNames:
                lastItemIdx = self.chooser.GetCount()

                for fileName in fileNames:
                    self.fileNames.append((directory, fileName))

                self._update_chooser()

                if len(fileNames) == 1 or lastItemIdx == 0:
                    self.chooser.SetSelection(lastItemIdx)
                    self._select_file(lastItemIdx)

        dlg.Destroy()

    def _update_chooser(self):
        chooser = self.chooser
        sel = self.chooser.GetSelection()
        skip = common_prefix([dir for (dir, fname) in self.fileNames])

        chooser.Clear()
        chooser.Append(self.NO_SELECTION)
        for directory, fileName in self.fileNames:
            chooser.Append(os.path.join(directory[skip:], fileName))

        chooser.SetSelection(sel)

    def _select_file(self, index):
        if index == 0:
            fileName = None
        else:
            fileName = os.path.join(*self.fileNames[index - 1])
        wx.PostEvent(self, FileChoiceEvent(self.GetId(), fileName))

    def SetToolTip(self, tooltip):
        self.chooser.SetToolTip(wx.ToolTip(tooltip.GetTip()))
        self.browser.SetToolTip(wx.ToolTip(tooltip.GetTip()))


def common_prefix(directories):
    def allsame(seq):
        return min([elem==seq[0] for elem in seq]+[True])

    if not directories:
        return ''

    dirparts = [x.split(os.sep) for x in directories]
    m = [allsame(seq) for seq in zip(*dirparts)]
    m.append(False)
    skip = len(os.sep.join(dirparts[0][0:m.index(False)]))
    if skip != 0: # strip the leading /
        skip += len(os.sep)
    return skip


class TestFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, -1, 'FileChoice')
        chooser1 = FileChoice(self, -1)
        chooser1.SetToolTip(wx.ToolTip('FileChoice 1'))
        chooser2 = FileChoice(self, -1, choices=os.listdir('.'))
        chooser2.SetToolTip(wx.ToolTip('FileChoice with initial choices'))

        szr = wx.BoxSizer(wx.VERTICAL)
        szr.Add(chooser1, 0,wx.EXPAND|wx.ALL, 3)
        szr.Add(chooser2, 0,wx.EXPAND|wx.ALL, 3)
        self.SetSizer(szr)

        EVT_FILE_CHOICE(self, chooser1.GetId(), self.OnFileChoice1)
        EVT_FILE_CHOICE(self, chooser2.GetId(), self.OnFileChoice2)

    def OnFileChoice1(self, evt):
        print 'Select 1:', repr(evt.fileName)

    def OnFileChoice2(self, evt):
        print 'Select 2:', repr(evt.fileName)


def main():
    app = wx.PySimpleApp()
    frame = TestFrame()
    frame.Show(True)
    app.MainLoop()


if __name__ == '__main__':
    main()
