# Name: libsetuproi/plot.py
# Purpose: ROI plotting widget
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006-2009 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

import wxmpl
from matplotlib import numerix as nx


class RoiPlotPanel(wxmpl.PlotPanel):
    def __init__(self, parent):
        wxmpl.PlotPanel.__init__(self, parent, -1)
        self.region  = None
        self.channel = None
        self.roiStart = None
        self.roiEnd   = None
        self.preEdgeSpectrum  = None
        self.postEdgeSpectrum = None

        self.zoom_limits = None
        self.axes = None

    def redraw(self):
        if self.preEdgeSpectrum is None and self.postEdgeSpectrum is None:
            self.get_figure().clear()
            self.draw()
            return

        # Setup the axes
        self.axes = axes = self.get_figure().gca()
        self.save_zoom_limits()
        axes.clear()
        axes.set_title('Channel %d, ROI %d'
            % (self.channel, self.region), fontsize=12)

        # Plot the ROI boundaries
        def plot_boundary(x, linestyle='--'):
            if x is not None:
                axes.axvline(x, linewidth=1, color='gray',
                    label='_nolegend_', linestyle=linestyle)

        plot_boundary(self.roiStart)
        plot_boundary(self.roiEnd)
        if self.roiStart is not None and self.roiEnd is not None:
            center = self.roiStart + (self.roiEnd - self.roiStart)/2.0
            plot_boundary(center, linestyle=':')

        # Plot the data
        # purple roi, green data
        self.plot_data(self.preEdgeSpectrum,  'purple',  '#22aa00', 'pre')
        # blue roi,   red data
        self.plot_data(self.postEdgeSpectrum, '#ff1133', '#0066cc', 'post')

        # Finish up
        axes.legend()
        self.restore_zoom_limits()
        self.draw()

    def plot_data(self, spectrum, roiColor, dataColor, labelPrefix):
        roiLabel = labelPrefix + ' roi'
        dataLabel = labelPrefix + ' data'

        def plot(x, y, color, label):
            self.axes.plot(x, y, label=label, linestyle='', marker='o',
                markersize=2.25, markeredgewidth=0.5, markerfacecolor=color,
                markeredgecolor=color)

        x_data, y_data, x_roi, y_roi = self.split_roi_from_data(spectrum)
        if x_data is None and x_roi is None:
            return
        elif x_data is None:
            plot(x_roi,  y_roi,  roiColor,  roiLabel)
        elif x_roi is None:
            plot(x_data, y_data, dataColor, dataLabel)
        else:
            plot(x_roi,  y_roi,  roiColor,  roiLabel)
            plot(x_data, y_data, dataColor, dataLabel)

    def save_zoom_limits(self):
        if self.zoomed(self.axes):
            # NB: MPL 0.98.1 returns the underlying array objects of the limits
            self.zoom_limits = (tuple(self.axes.get_xlim()),
                tuple(self.axes.get_ylim()))

    def restore_zoom_limits(self):
        if self.zoom_limits is not None:
            xlim, ylim = self.zoom_limits
            self.zoom_limits = None
            self.axes.set_xlim(xlim)
            self.axes.set_ylim(ylim)

    def split_roi_from_data(self, spectrum):
        if spectrum is None:
            return None, None, None, None
        elif self.roiStart is None or self.roiEnd is None:
            x_data = nx.arange(0, spectrum.shape[0], 1.0)
            return x_data, spectrum, None, None

        start = max(self.roiStart, 0)
        stop  = min(self.roiEnd, spectrum.shape[0])
        start, stop = min(start, stop), max(start, stop)

        preData = spectrum[:start]
        pstData = spectrum[stop:]
        y_roi   = spectrum[start:stop]
        x_roi = nx.arange(start, stop, 1.0)

        n = preData.shape[0] + pstData.shape[0]
        if n == 0:
            return None, None, x_roi, y_roi

        i = preData.shape[0]
        x_data = nx.zeros((n,), nx.Float)
        x_data[:i] = nx.arange(0, start, 1.0)
        x_data[i:] = nx.arange(stop, spectrum.shape[0], 1.0)
        y_data = nx.zeros((n,), nx.Float)
        y_data[:i] = preData
        y_data[i:] = pstData

        if y_roi.shape[0] == 0:
            return x_data, y_data, None, None
        else:
            return x_data, y_data, x_roi, y_roi


