# Name: libsetuproi/roiset.py
# Purpose: Class which contains a list of regions of interest for each channel
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006-2009 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

import math
from matplotlib import numerix as nx

__all__ = ['RoiList']

NUM_CHANNELS = 19 # detector channels
NUM_REGIONS  = 16 # ROIs per channel


def make_roi(start, end):
    """
    Converts the numbers I{start} and I{end} to an ROI tuple.
    """
    if start is not None:
        start = int(start)

    if end is not None:
        end = int(math.ceil(end))

    return start, end


class RoiList(object):
    """
    A list of regions of interest for each channel on a multichannel analyzer.
    """

    def __init__(self):
        self.rois = nx.zeros((NUM_REGIONS, NUM_CHANNELS, 2),
                        nx.PyObject)
        self.rois[:] = None

    def getROI(self, regionNumber, channelNumber):
        """
        Gets the ROI tuple I{(start, end)} of a channel's region.
        """
        if -1 < regionNumber < NUM_REGIONS:
            if -1 < channelNumber < NUM_CHANNELS:
                return tuple(self.rois[regionNumber, channelNumber])
            else:
                raise ValueError('invalid channel number %s' % channelNumber)
        else:
            raise ValueError('invalid region number %s' % regionNumber)

    def setROI(self, regionNumber, channelNumber, start, end):
        """
        Sets the ROI of a channel's region.
        """
        if -1 < regionNumber < NUM_REGIONS:
            if -1 < channelNumber < NUM_CHANNELS:
                self.rois[regionNumber, channelNumber] = make_roi(start, end)
            else:
                raise ValueError('invalid channel number %s' % channelNumber)
        else:
            raise ValueError('invalid region number %s' % regionNumber)

    def getChannelROIs(self, regionNumber):
        """
        Gets the list of channels' ROI tuples for a region.
        """
        if -1 < regionNumber < NUM_REGIONS:
            return [tuple(self.rois[regionNumber, x])
                    for x in range(0, NUM_CHANNELS)]
        else:
            raise ValueError('invalid region number %s' % regionNumber)

    def isEmpty(self):
        """
        Tests whether this ROI list contains any non-None ROI tuples.
        """
        for i in range(0, NUM_REGIONS):
            for j in range(0, NUM_CHANNELS):
                if self.rois[i,j][0] != None and self.rois[i,j][1] != None:
                    return False
        return True

    def toMotorScript(self):
        """
        Converts the ROI list to an MX motor script.
        """
        s = ''
        for reg in range(0, NUM_REGIONS):
            for chan in range(0, NUM_CHANNELS):
                start, end = self.getROI(reg, chan)
                if start is not None and end is not None:
                    s += ('mca mcaxia%d set roi %d %d %d\n'
                        % (chan, reg, start, end))
        return s

    def fromMotorScript(cls, script):
        """
        Creates a new ROI list from an MX motor script.
        """
        rl = cls()
        for line in script.split('\n'):
            if not line.startswith('mca mcaxia'):
                continue

            line = line[10:].split()
            if len(line) != 6:
                continue

            line = line[0:1] + line[3:]
            try:
                channel, region, start, end = [int(x) for x in line]
                rl.setROI(region, channel, start, end)
            except ValueError:
                continue
        return rl
    fromMotorScript = classmethod(fromMotorScript)
