# Name: libsetuproi/togglebox.py
# Purpose: a box of toggle buttons that work like radio buttons
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.


import os
import os.path
import wx

__all__ = ['ToggleBox', 'EVT_TOGGLEBOX']


#
# wxPython event generated by the ToggleBox widget
#

EVT_TOGGLEBOX_ID = wx.NewId()


def EVT_TOGGLEBOX(win, id, func):
    """
    Register to receieve selection events from a ToggleBox.
    """
    win.Connect(id, -1, EVT_TOGGLEBOX_ID, func)


class ToggleBoxEvent(wx.PyCommandEvent):
    """
    wxPython event emitted when the selection of a ToggleBox is changed.
    """

    def __init__(self, id, index):
        wx.PyCommandEvent.__init__(self, EVT_TOGGLEBOX_ID, id)
        self.SetInt(index)
        self.SetExtraLong(True)

    def Clone(self):
        return ToggleBoxEvent(self.GetId(), self.index)


class ToggleBox(wx.Panel):
    def __init__ (self, parent, id, rows, columns, *args, **kwds):
        wx.Panel.__init__ (self, parent, id, *args, **kwds)
        self.toggleButtons = []
        self.selectedButtonIndex = None

        for i in range(0, rows):
            for j in range(0, columns):
                btn = wx.ToggleButton(self, -1, str(i*columns+j),
                    size=wx.Size(25, 25))
                btn.SetFont(wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.NORMAL))
                self.toggleButtons.append(btn)
                wx.EVT_TOGGLEBUTTON(self, btn.GetId(), self._on_select_button)
        self._toggle_button(0)

        szr = wx.GridSizer(rows, columns)
        for btn in self.toggleButtons:
            szr.Add(btn, 0, 0, 0)
        pad = wx.BoxSizer(wx.VERTICAL)
        pad.Add(szr, 0, wx.ALL, 0)
        self.SetSizer(pad)
        self.Fit()

    def _toggle_button(self, selection):
        if self.selectedButtonIndex is not None:
            btn = self.toggleButtons[self.selectedButtonIndex]
            btn.SetValue(False)
            btn.SetBackgroundColour(
                wx.SystemSettings_GetColour(wx.SYS_COLOUR_BTNFACE))
            btn.SetForegroundColour(
                wx.SystemSettings_GetColour(wx.SYS_COLOUR_BTNTEXT))

        btn = self.toggleButtons[selection]
        btn.SetBackgroundColour(wx.Colour(255, 255, 255))
        btn.SetForegroundColour(wx.Colour(  0,   0, 255))
        btn.SetValue(True)
        self.selectedButtonIndex = selection

    def _on_select_button(self, evt):
        index = self.toggleButtons.index(evt.GetEventObject())
        self._toggle_button(index)
        wx.PostEvent(self, ToggleBoxEvent(self.GetId(), index))

    def GetSelection(self):
        return self.selectedButtonIndex

    def SetSelection(self, index):
        if 0 <= index < len(self.toggleButtons):
            self._toggle_button(index)
        else:
            self._toggle_button(0)

    def SetToolTip(self, tooltip):
        for btn in self.toggleButtons:
            btn.SetToolTip(wx.ToolTip(tooltip.GetTip()))


class TestFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, -1, 'ToggleBox')
        toggle = ToggleBox(self, -1, 2, 8)

        szr = wx.BoxSizer(wx.VERTICAL)
        szr.Add(toggle, 0, wx.ALL, 3)
        self.SetSizer(szr)

        EVT_TOGGLEBOX(self, toggle.GetId(), self.OnToggleBox)

    def OnToggleBox(self, evt):
        print 'Select:', evt.GetSelection()


def main():
    app = wx.PySimpleApp()
    frame = TestFrame()
    frame.Show(True)
    app.MainLoop()


if __name__ == '__main__':
    main()
