# Name: libsetuproi/view.py
# Purpose: Interface for the View portion of the MVP triad.
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

import sys

__all__ = ['View', 'MockView']


class View:
    """
    Base class for View classes in the MVP triad.
    """

    def showErrorMessage(self, title, message, *args):
        """
        Displays an error dialog.
        """

    def showWarningMessage(self, title, message, *args):
        """
        Displays a warning dialog.
        """

    def showInfoMessage(self, title, message, *args):
        """
        Displays an informational dialog.
        """

    def askOkCancel(self, title, message, *args):
        """
        Displays an Ok/Cancel dialog and returns a boolean.
        """

    def askFileOpen(self, title, glob='All files (*.*)|*', directory=''):
        """
        Displays an "open file" dialog and returns the full filename, or None
        if the user cancelled.
        """

    def askFileOpenMany(self, title, glob='All files (*.*)|*', directory=''):
        """
        Displays an "open multiple files" dialog and returns the list of
        selections, or None if the user cancelled.
        """

    def askFileSave(self, title, glob='All files (*.*)|*', directory=''):
        """
        Displays a "save file" dialog and returns the full filename, or None if
        the user cancelled.
        """


class MockView(View):
    """
    View which is suitable for use as a mock object in unit tests.
    """

    UNSET = ('UNSET',)

    def _format_message(self, title, message, args):
        return '[%s] %s' % (title, message % args)

    errorMessage = UNSET
    def showErrorMessage(self, title, message, *args):
        self.errorMessage = self._format_message(title, message, args)

    warningMessage = UNSET
    def showWarningMessage(self, title, message, *args):
        self.warningMessage = self._format_message(title, message, args)

    infoMessage = UNSET
    def showInfoMessage(self, title, message, *args):
        self.infoMessage = self._format_message(title, message, args)

    okCancelValue = False
    wasAskedOkCancel = False
    def askOkCancel(self, title, message, *args):
        """
        Returns the okCancelValue attribute.
        """
        self.wasAskedOkCancel = True
        return self.okCancelValue

    fileOpenValue = None
    wasAskedFileOpen = False
    def askFileOpen(self, title, glob='All files (*.*)|*', directory=''):
        """
        Returns the fileOpenValue attribute.
        """
        self.wasAskedFileOpen = True
        return self.fileOpenValue

    fileOpenManyValue = None
    wasAskedFileOpenMany = False
    def askFileOpenMany(self, title, glob='All files (*.*)|*', directory=''):
        """
        Returns the fileOpenManyValue attribute.
        """
        self.wasAskedFileOpenMany = True
        return self.fileOpenManyValue

    fileSaveValue = None
    wasAskedFileSave = False
    def askFileSave(self, title, glob='All files (*.*)|*', directory=''):
        """
        Returns the fileSaveValue attribute.
        """
        self.wasAskedFileSave = True
        return self.fileSaveValue
