# Name: libsetuproi/wxview.py
# Purpose: wxPython-based view classes.
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

import wx

__all__ = ['NullView', 'FrameView']


def _construct_window_title(prefix, name):
    """
    Creates a string suitable for use as a window title from two other strings.
    """
    prefix = prefix or ''
    prefix = prefix.strip()
    name = name.strip()

    if not prefix:
        if not name:
            return 'Untitled'
        else:
            return name
    else:
        if not name:
            return prefix
        else:
            return '%s - %s' % (prefix, name)


class WxView:
    """
    Base class for wxPython view classes.
    """

    windowName = 'wxPython'
    dialogName = 'wxPython'

    def make_title(self, prefix):
        """
        Creates a title string from the provided I{prefix} and I{windowName}.
        """
        return _construct_window_title(prefix, self.windowName)

    def make_dialog_title(self, prefix):
        """
        Creates a title string from the provided I{prefix} and I{dialogName}.
        """
        return _construct_window_title(prefix, self.dialogName)

    def _get_dialog_parent(self):
        """
        Private template method, used by NullView to override the parent
        argument passed to MessageBoxes.
        """
        return self

    def _display_message_box(self, title, message, args, style):
        """
        Private method that displays a message dialog.
        """
        return wx.MessageBox(message % args, self.make_dialog_title(title),
            style, parent=self._get_dialog_parent())

    def _display_file_dialog(self, title, glob, directory, style):
        """
        Private method that display a file dialog and returns it or None
        if the user cancelled.
        """
        dlg = wx.FileDialog(self._get_dialog_parent(), message=title,
            wildcard=glob, defaultDir=directory, style=style)
        if dlg.ShowModal() == wx.ID_OK:
            dlg.Destroy()
            return dlg
        else:
            dlg.Destroy()
            return None

    def showErrorMessage(self, title, message, *args):
        """
        Displays an error dialog.
        """
        self._display_message_box(title, message, args, wx.OK|wx.ICON_ERROR)

    def showWarningMessage(self, title, message, *args):
        """
        Displays a warning dialog.
        """
        self._display_message_box(title, message, args,
            wx.OK|wx.ICON_EXCLAMATION)

    def showInfoMessage(self, title, message, *args):
        """
        Displays an informational dialog.
        """
        self._display_message_box(title, message, args,
            wx.OK|wx.ICON_INFORMATION)

    def askOkCancel(self, title, message, *args):
        """
        Displays an Ok/Cancel dialog and returns a boolean.
        """
        return wx.OK == self._display_message_box(title, message, args,
            wx.OK|wx.CANCEL|wx.ICON_QUESTION)

    def askFileOpen(self, title, glob='All files (*.*)|*', directory=''):
        """
        Displays an "open file" dialog and returns the full filename, or None
        if the user cancelled.
        """
        dlg = self._display_file_dialog(title, glob, directory, wx.OPEN)
        if dlg is None: return None
        else:           return dlg.GetPath()

    def askFileOpenMany(self, title, glob='All files (*.*)|*', directory=''):
        """
        Displays an "open multiple files" dialog and returns the list of
        selections, or None if the user cancelled.
        """
        dlg = self._display_file_dialog(title, glob, directory,
            wx.OPEN|wx.MULTIPLE)
        if dlg is None: return None
        else:           return dlg.GetPaths()

    def askFileSave(self, title, glob='All files (*.*)|*', directory=''):
        """
        Displays a "save file" dialog and returns the full filename, or None if
        the user cancelled.
        """
        dlg = self._display_file_dialog(title, glob, directory,
            wx.SAVE|wx.OVERWRITE_PROMPT,)
        if dlg is None: return None
        else:           return dlg.GetPath()


class NullView(WxView):
    """
    View which is not a wxPython widget but can still display dialogs.
    """
    def _get_dialog_parent(self):
        return None


class FrameView(WxView, wx.Frame):
    """
    View which is also a wxPython Frame.
    """

    def __init__(self, parent, *args, **kwds):
        wx.Frame.__init__(self, parent, -1, '', *args, **kwds)
        self.updateTitle(None)

    def updateTitle(self, prefix):
        """
        Updates the frame title using make_title() and I{prefix}.
        """
        self.SetTitle(self.make_title(prefix))

