#!/usr/bin/env python
# Name: test_roi.py
# Purpose: Unit tests for libsetuproi.command
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

import unittest
from libsetuproi.command import *


class AppendCommand(Command):
    DESCRIPTION = 'Append to list'

    def __init__(self, list, value):
        self.list = list
        self.value = value

    def execute(self):
        memento = self.list[:]
        self.list.append(self.value)
        return memento

    def undo(self, memento):
        self.list[:] = memento


class AppendCommandThatCantUndo(Command):
    DESCRIPTION = 'Append to list forever'

    def __init__(self, list, value):
        self.list = list
        self.value = value

    def execute(self):
        self.list.append(self.value)


class AppendCommandThatFails(Command):
    def __init__(self, list, value):
        self.list = list
        self.value = value

    def execute(self):
        raise CommandError('AppendCommandThatFails', 'I told you so!')


class AppendCommandThatCantRedo(AppendCommand):
    def __init__(self, list, value):
        AppendCommand.__init__(self, list, value)
        self.done = False

    def execute(self):
        if self.done:
            raise CommandError('AppendCommandThatCantRedo', 'Nein!')
        else:
            self.done = True
            return AppendCommand.execute(self)


class TestCommand(unittest.TestCase):
    def test_creation(self):
        L = []
        cr = CommandRunner()
        cmds = [AppendCommand(L, x) for x in range(0,5)]

    def test_canUndo(self):
        cr = CommandRunner()
        self.assert_(not cr.canUndo)
        cr.execute(AppendCommand([], 0))
        self.assert_(cr.canUndo)
        cr.execute(AppendCommandThatCantUndo([], 0))
        self.assert_(not cr.canUndo)

    def test_undo(self):
        L = []
        cr = CommandRunner()
        cr.execute(AppendCommand(L, 0))
        self.assertEquals(L, [0])
        self.assert_(cr.canUndo)

        self.assert_(cr.undo())
        self.assertEquals(L, [])
        self.assert_(not cr.canUndo)

    def test_canRestore(self):
        cr = CommandRunner()
        self.assert_(not cr.canRestore)
        cr.execute(AppendCommand([], 0))
        self.assert_(not cr.canRestore)
        cr.undo()
        self.assert_(cr.canRestore)

        cr.execute(AppendCommandThatCantUndo([], 0))
        self.assert_(not cr.canRestore)

    def test_restore(self):
        L = []
        cr = CommandRunner()
        cr.execute(AppendCommand(L, 0))
        cr.undo()
        self.assert_(cr.restore())
        self.assertEquals(L, [0])
        self.assert_(cr.canUndo)
        self.assert_(not cr.canRestore)

    def test_deep_undo_restore(self):
        L = []
        cr = CommandRunner()
        for i, cmd in enumerate([AppendCommand(L, x) for x in range(0,5)]):
            cr.execute(cmd)
            self.assertEquals(L, range(i + 1))

        for i in range(0, 5):
            self.assert_(cr.canUndo)
            self.assert_(cr.undo())

        for i in range(0, 5):
            self.assert_(cr.canRestore)
            self.assert_(cr.restore())

        self.assert_(cr.canUndo)
        self.assert_(not cr.canRestore)

    def test_history_undoable_command(self):
        L = []
        cr = CommandRunner()
        for i, cmd in enumerate([AppendCommand(L, x) for x in range(0,5)]):
            cr.execute(cmd)

        for i in range(0, 5):
            self.assert_(cr.undo())

        cr.execute(AppendCommand(L, 100))
        self.assertEquals(1, len(cr.history))        

    def test_history_nonundoable_command(self):
        L = []
        cr = CommandRunner()
        for i, cmd in enumerate([AppendCommand(L, x) for x in range(0,5)]):
            cr.execute(cmd)

        for i in range(0, 5):
            self.assert_(cr.undo())

        cr.execute(AppendCommandThatCantUndo(L, 100))
        self.assertEquals(1, len(cr.history))        

    def test_command_failure(self):
        L = []
        cr = CommandRunner()
        self.assertEquals(False, cr.execute(AppendCommandThatFails(L, 0)))
        self.assertEquals(0, len(cr.history))

    def test_restore_failure(self):
        L = []
        cr = CommandRunner()
        self.assert_(cr.execute(AppendCommandThatCantRedo(L, 0)))
        self.assert_(cr.undo())
        self.assertEquals(False, cr.restore())
        self.assertEquals(0, len(cr.history))

    def test_dirty(self):
        L = []
        cr = CommandRunner()
        self.assertEquals(False, cr.dirty)

        cr.execute(AppendCommand(L, 0))
        self.assertEquals(True, cr.dirty)

        cr.undo()
        self.assertEquals(False, cr.dirty)

        cr.restore()
        self.assertEquals(True, cr.dirty)

        cr.execute(AppendCommand(L, 0))
        self.assertEquals(True, cr.dirty)

        cr.undo()
        self.assertEquals(True, cr.dirty)
        cr.undo()
        self.assertEquals(False, cr.dirty)

        cr.restore()
        self.assertEquals(True, cr.dirty)
        cr.restore()
        self.assertEquals(True, cr.dirty)

        cr.dirty = False
        self.assertEquals(False, cr.dirty)
        cr.undo()
        self.assertEquals(True, cr.dirty)
        cr.restore()
        self.assertEquals(False, cr.dirty)

        cr.dirty = True
        self.assertEquals(True, cr.dirty)
        cr.undo()
        self.assertEquals(True, cr.dirty)
        cr.undo()
        self.assertEquals(False, cr.dirty)

    def test_undoDescription(self):
        L = []
        cr = CommandRunner()
        self.assertEquals(None, cr.undoDescription)

        cr.execute(AppendCommand(L, 0))
        self.assertEquals(AppendCommand.DESCRIPTION, cr.undoDescription)

        cr.undo()
        self.assertEquals(None, cr.undoDescription)

        cr.restore()
        self.assertEquals(AppendCommand.DESCRIPTION, cr.undoDescription)

    def test_restoreDescription(self):
        L = []
        cr = CommandRunner()
        self.assertEquals(None, cr.restoreDescription)

        cr.execute(AppendCommand(L, 0))
        self.assertEquals(None, cr.restoreDescription)

        cr.undo()
        self.assertEquals(AppendCommand.DESCRIPTION, cr.restoreDescription)

        cr.restore()
        self.assertEquals(None, cr.restoreDescription)


if __name__ == '__main__':
    unittest.main()
