# Name: test_presenter.py
# Purpose: Unit tests for libsetuproi.presenter.SetupRoiPresenter
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

import Numeric
import unittest
import libsetuproi.view
from libsetuproi.director import *
from libsetuproi.presenter import *


class MockView(libsetuproi.view.MockView):
    UNSET = libsetuproi.view.MockView.UNSET

    def __init__(self, parent=None, director=None):
        if director is None:
            director = AppDirector(self, MockView)

        self.director  = director
        self.presenter = SetupRoiPresenter(director, self)

    # Director API
    def canClose(self):
        return self.presenter.canClose()

    def Show(self, enabled):
        pass

    # General presenter API

    undoDescription = UNSET
    restoreDescription = UNSET
    def updateUndoRestore(self, undoDescription, restoreDescription):
        self.undoDescription = undoDescription
        self.restoreDescription = restoreDescription

    fileName = UNSET
    def setFileName(self, fileName):
        self.fileName = fileName

    # Plotting Data

    preEdgeSpectrum = UNSET
    def setPreEdgeSpectrum(self, spectrum):
        self.preEdgeSpectrum = spectrum

    postEdgeSpectrum = UNSET
    def setPostEdgeSpectrum(self, spectrum):
        self.postEdgeSpectrum = spectrum

    plotSelectionValue = UNSET
    def setPlotSelection(self, region, channel):
        self.plotSelectionValue = region, channel

    plotRoiRange = UNSET
    def setPlotRoiRange(self, range):
        self.plotRoiRange = range

    # Plotting API

    updatePlotCalled = UNSET
    numberUpdatePlotCalls = 0 
    def updatePlot(self):
        self.updatePlotCalled = True
        self.numberUpdatePlotCalls += 1

    # Region Of Interest Editor

    roiValue = UNSET
    roiValueDict = {}
    numberSetRoiCalls = 0
    def setRoi(self, region, channel, roi):
        self.numberSetRoiCalls += 1
        self.roiValue = region, channel, roi
        self.roiValueDict[(region, channel)] = roi

    regionSelectionValue = UNSET
    def setRegionSelection(self, region):
        self.regionSelectionValue = region

    channelSelectionValue = UNSET
    def setChannelSelection(self, channel, column):
        self.channelSelectionValue = channel, column


EMPTY_ROI = '', ''
EMPTY_PLOT_ROI = None, None

class TestPresenterSelection(unittest.TestCase):
    def test_select_region(self):
        v = MockView(); p = v.presenter
        p.selectRegion(1)
        self.assertEquals((1, 18, EMPTY_ROI), v.roiValue)
        self.assertEquals(19, v.numberSetRoiCalls)
        self.assertEquals((1, 0), v.plotSelectionValue)
        self.assertEquals(EMPTY_PLOT_ROI, v.plotRoiRange)
        self.assertEquals(True, v.updatePlotCalled)
        self.assertEquals(1, v.numberUpdatePlotCalls)

    def test_select_region_twice(self):
        v = MockView(); p = v.presenter
        p.selectRegion(1)
        v.updatePlotCalled = False
        v.roiValue = v.plotRoiRange = None # ensure that these are not updated
        v.numberSetRoiCalls = v.numberUpdatePlotCalls = 0

        p.selectRegion(1)
        self.assertEquals(None, v.roiValue)
        self.assertEquals(0, v.numberSetRoiCalls)
        self.assertEquals(None, v.plotRoiRange)
        self.assertEquals(False, v.updatePlotCalled)
        self.assertEquals(0, v.numberUpdatePlotCalls)

    def test_select_channel(self):
        v = MockView(); p = v.presenter
        p.selectChannel(1, 0)
        self.assertEquals((0, 1, EMPTY_ROI), v.roiValue)
        self.assertEquals((0, 1), v.plotSelectionValue)
        self.assertEquals(EMPTY_PLOT_ROI, v.plotRoiRange)
        self.assertEquals(None, v.preEdgeSpectrum)
        self.assertEquals(None, v.postEdgeSpectrum)
        self.assertEquals(True, v.updatePlotCalled)
        self.assertEquals(1, v.numberUpdatePlotCalls)

    def test_select_channel_twice(self):
        v = MockView(); p = v.presenter
        p.selectChannel(1, 0)
        v.updatePlotCalled = False
        v.roiValue = v.plotRoiRange = None
        v.preEdgeSpectrum = v.postEdgeSpectrum = v.UNSET
        v.numberSetRoiCalls = v.numberRedrawPlotCalls = 0

        p.selectChannel(1, 0)
        self.assertEquals(None, v.roiValue)
        self.assertEquals(None, v.plotRoiRange)
        self.assertEquals(v.UNSET, v.preEdgeSpectrum)
        self.assertEquals(v.UNSET, v.postEdgeSpectrum)
        self.assertEquals(False, v.updatePlotCalled)


class TestPresenterEditRoi(unittest.TestCase):
    from libsetuproi.presenter import ModifyRoiCommand
    D = ModifyRoiCommand.DESCRIPTION

    # ROI start arguments to check_view()
    START_BEFORE = 0, 0, 0, None, D, EMPTY_ROI, EMPTY_PLOT_ROI
    START_AFTER  = 0, 0, 0, D, None, ('1', ''), (1, None)

    # ROI end arguments to check_view()
    END_BEFORE = 0, 0, 1, None, D, EMPTY_ROI, EMPTY_PLOT_ROI
    END_AFTER  = 0, 0, 1, D, None, ('', '1'), (None, 1)

    def check_view(self, v, (region, channel, column, undo, restore, roiValue,
     plotRoiRange)):
        self.assertEqual(undo, v.undoDescription)
        self.assertEqual(restore, v.restoreDescription)
        self.assertEquals((region, channel, roiValue), v.roiValue)
        self.assertEquals(plotRoiRange, v.plotRoiRange)
        self.assertEquals(True, v.updatePlotCalled)

    def make_presenter(self, before, after):
        v = MockView(); p = v.presenter
        p.selectChannel(before[1], before[2])
        p.editRoi('1')
        self.check_view(v, after)
        return p, v

    def check_edit_region(self, before, after):
        p, v = self.make_presenter(before, after)

    def check_edit_region_undo(self, before, after):
        p, v = self.make_presenter(before, after)

        p.undoLastCommand()
        self.check_view(v, before)

    def check_edit_region_restore(self, before, after):
        p, v = self.make_presenter(before, after)

        p.undoLastCommand()
        self.check_view(v, before)

        p.restoreLastCommand()
        self.check_view(v, after)

    def test_edit_start(self):
        self.check_edit_region(self.START_BEFORE, self.START_AFTER)

    def test_edit_start_undo(self):
        self.check_edit_region_undo(self.START_BEFORE, self.START_AFTER)

    def test_edit_start_restore(self):
        self.check_edit_region_restore(self.START_BEFORE, self.START_AFTER)

    def test_edit_end(self):
        self.check_edit_region(self.END_BEFORE, self.END_AFTER)

    def test_edit_end_undo(self):
        self.check_edit_region_undo(self.END_BEFORE, self.END_AFTER)

    def test_edit_end_restore(self):
        self.check_edit_region_restore(self.END_BEFORE, self.END_AFTER)


class TestPresenterCanClose(unittest.TestCase):
    def setUp(self):
        v = self.view = MockView()
        p = self.presenter = v.presenter
        self.assertEquals(True, p.canClose())

        p.editRoi('1')
        p.selectChannel(0, 1)
        p.editRoi('2')
        self.assertEquals(False, p.canClose())

    def test_can_close(self):
        pass # setUp() does it all

    def test_undo_command(self):
        p = self.presenter
        p.undoLastCommand()
        self.assertEquals(True, p.canClose())

    def test_restore_command(self):
        p = self.presenter
        p.undoLastCommand()
        self.assertEquals(True, p.canClose())
        p.restoreLastCommand()
        self.assertEquals(False, p.canClose())

# TODO: delete this
#    def test_clear_undo_history(self):
#        p = self.presenter
#        p.clearUndoHistory()
#        self.assertEquals(True, p.canClose())


class TestPresenterCloseFrame(unittest.TestCase):
    def test_for_can_close(self):
        v = MockView() ; p = v.presenter
        self.assertEquals(True, p.closeFrame())
        self.assertEquals(False, v.wasAskedOkCancel)

    def test_for_cannot_close(self):
        v = MockView() ; p = v.presenter

        p.editRoi('1')
        self.assertEquals(True, p.closeFrame())
        self.assertEquals(False, v.wasAskedOkCancel)

        p.editRoi('1')
        p.selectChannel(0, 1)
        p.editRoi('10')
        v.okCancelValue = True
        v.wasAskedOkCancel = False
        self.assertEquals(True, p.closeFrame())
        self.assertEquals(True, v.wasAskedOkCancel)


class TestPresenterEditRoiErrors(unittest.TestCase):
    def setUp(self):
        self.view = MockView()
        self.presenter = self.view.presenter

    def edit(self, column, value):
        self.presenter.selectChannel(0, column)
        self.presenter.editRoi(value)

    def assertError(self, column, value):
        self.edit(column, value)
        self.assertNotEqual(self.view.UNSET, self.view.errorMessage)

    def test_not_number(self):
        for value in ['a', '-1.2a', '@#@fdfd34']:
            for column in 0, 1:
                self.assertError(column, value)

    def test_negative_number(self):
        for column in 0, 1:
            self.assertError(column, '-1')

    def test_end_too_small(self):
        self.edit(0, '100')
        self.assertError(1, '50')


class TestNewFrame(unittest.TestCase):
    def test_director_create(self):
        v = MockView()
        v.director.createNewFrame()


class TestPresenterRoiFileBase(unittest.TestCase):
    VALUES = (0, 0, ('1', '2')),  (0, 1, ('2', '3')),  (0, 2, ('3', '4'))
    PLOT_VALUES = (1, 2), (2, 3), (3, 4)

    def setUp(self):
        import os
        import tempfile
        script = ('mca mcaxia0 set roi 0 1 2\n'
                  'mca mcaxia1 set roi 0 2 3\n'
                  'mca mcaxia2 set roi 0 3 4\n')
        fd1, self.tempName = tempfile.mkstemp()
        fd2, self.scriptName = tempfile.mkstemp()
        os.close(fd1)
        os.write(fd2, script)
        os.close(fd2)
        self.baseTempName = os.path.basename(self.tempName)
        self.baseScriptName = os.path.basename(self.scriptName)

    def tearDown(self):
        import os
        for fname in self.tempName, self.scriptName:
            try:
                os.unlink(fname)
            except:
                pass


class TestPresenterOpenRoiList(TestPresenterRoiFileBase):
    def test_open_canceled(self):
        v = MockView(); p = v.presenter

        p.openRoiList()
        self.assertEquals(True, v.wasAskedFileOpen)
        self.assertEquals(v.UNSET, v.fileName)

    def check_exec(self, v):
        from libsetuproi.presenter import LoadRoiCommand
        self.assertEqual(LoadRoiCommand.DESCRIPTION, v.undoDescription)
        self.assertEqual(None, v.restoreDescription)
        self.assertEquals(True, v.wasAskedFileOpen)
        self.assertEquals(self.baseScriptName, v.fileName)
        for region, channel, roi in self.VALUES:
            self.assertEquals(roi, v.roiValueDict[(region, channel)])
        self.assertEquals(self.PLOT_VALUES[0], v.plotRoiRange)

    def check_undo(self, v):
        from libsetuproi.presenter import LoadRoiCommand
        self.assertEqual(None, v.undoDescription)
        self.assertEqual(LoadRoiCommand.DESCRIPTION, v.restoreDescription)
        self.assertEquals(None, v.fileName)
        for region, channel, roi in self.VALUES:
            self.assertEquals(EMPTY_ROI, v.roiValueDict[(region, channel)])
        self.assertEquals(EMPTY_PLOT_ROI, v.plotRoiRange)

    def test_open(self):
        v = MockView(); p = v.presenter
        v.fileOpenValue = self.scriptName

        p.openRoiList()
        self.check_exec(v)

        p.undoLastCommand()
        self.check_undo(v)

        p.restoreLastCommand()
        self.check_exec(v)


class TestPresenterSaveRoiList(TestPresenterRoiFileBase):
    def test_save_canceled(self):
        v = MockView(); p = v.presenter

        p.saveRoiList()
        self.assertEquals(True, v.wasAskedFileSave)
        self.assertEquals(v.UNSET, v.fileName)

    def test_save_with_filename(self):
        v = MockView(); p = v.presenter
        p.set_file_name(self.tempName)

        p.saveRoiList()
        self.assertEquals(False, v.wasAskedFileSave)
        self.assertEquals(self.baseTempName, v.fileName)

    def test_save_without_filename(self):
        v = MockView(); p = v.presenter
        p.set_file_name(None)
        v.fileSaveValue = self.tempName

        p.saveRoiList()
        self.assertEquals(True, v.wasAskedFileSave)
        self.assertEquals(self.baseTempName, v.fileName)


class TestPresenterCanSave(TestPresenterRoiFileBase):
    def test_can_save_empty(self):
        v = MockView(); p = v.presenter
        self.assertEquals(False, p.canSave())

    def test_can_save_half_empty(self):
        v = MockView(); p = v.presenter
        p.editRoi('1')
        self.assertEquals(False, p.canSave())

    def test_can_save_not_empty(self):
        v = MockView(); p = v.presenter
        p.editRoi('1')
        p.selectChannel(0, 1)
        p.editRoi('10')
        self.assertEquals(True, p.canSave())


class TestPresenterSpectraFileBase(unittest.TestCase):
    SPECTRA = ((0, 4, 8, 12),
              (1, 5, 9, 13),
              (2, 6, 10, 14),
              (3, 7, 11, 15))

    def setUp(self):
        import os
        import tempfile
        text = (' .0  1  2  3\n'
                '  4  5  6  7\n'
                '  8  9 10 11\n'
                ' 12 13 14 15\n')
        fd, self.spectraFileName = tempfile.mkstemp()
        os.write(fd, text)
        os.close(fd)

    def tearDown(self):
        import os
        try:
            os.unlink(self.spectraFileName)
        except:
            pass

class TestPresenterSelectSpectra(TestPresenterSpectraFileBase):
    def test_select_pre(self):
        v = MockView(); p = v.presenter
        p.selectPreEdgeSpectra(self.spectraFileName)
        self.assertEquals(self.SPECTRA[0], tuple(v.preEdgeSpectrum))
        self.assertEquals(True, v.updatePlotCalled)

    def test_select_post(self):
        v = MockView(); p = v.presenter
        p.selectPostEdgeSpectra(self.spectraFileName)
        self.assertEquals(self.SPECTRA[0], tuple(v.postEdgeSpectrum))
        self.assertEquals(True, v.updatePlotCalled)



class TestPresenterSelectChannelWithSpectra(TestPresenterSpectraFileBase):
    def test_without_spectra(self):
        v = MockView(); p = v.presenter
        p.selectChannel(1, 0)
        self.assertEquals(None, v.preEdgeSpectrum)
        self.assertEquals(None, v.postEdgeSpectrum)
        self.assertEquals(True, v.updatePlotCalled)

    def test_with_pre_edge_spectra(self):
        v = MockView(); p = v.presenter
        p.selectPreEdgeSpectra(self.spectraFileName)
        p.selectChannel(1, 0)
        self.assertEquals(self.SPECTRA[1], tuple(v.preEdgeSpectrum))
        self.assertEquals(None, v.postEdgeSpectrum)
        self.assertEquals(True, v.updatePlotCalled)

    def test_with_post_edge_spectra(self):
        v = MockView(); p = v.presenter
        p.selectPostEdgeSpectra(self.spectraFileName)
        p.selectChannel(1, 0)
        self.assertEquals(None, v.preEdgeSpectrum)
        self.assertEquals(self.SPECTRA[1], tuple(v.postEdgeSpectrum))
        self.assertEquals(True, v.updatePlotCalled)

    def test_repeated_pre_edge_selection(self):
        v = MockView(); p = v.presenter
        p.selectPreEdgeSpectra(self.spectraFileName)

        v.preEdgeSpectrum = v.updatePlotCalled = None
        p.selectPreEdgeSpectra(self.spectraFileName)
        self.assertEquals(None, v.preEdgeSpectrum)
        self.assertEquals(None, v.updatePlotCalled)

    def test_repeated_post_edge_selection(self):
        v = MockView(); p = v.presenter
        p.selectPostEdgeSpectra(self.spectraFileName)

        v.postEdgeSpectrum = v.updatePlotCalled = None
        p.selectPostEdgeSpectra(self.spectraFileName)
        self.assertEquals(None, v.postEdgeSpectrum)
        self.assertEquals(None, v.updatePlotCalled)

    def test_null_pre_edge_spectra(self):
        v = MockView(); p = v.presenter
        p.selectPreEdgeSpectra(self.spectraFileName)
        p.selectPreEdgeSpectra(None)
        self.assertEquals(None, v.preEdgeSpectrum)
        self.assertEquals(True, v.updatePlotCalled)

    def test_null_post_edge_spectra(self):
        v = MockView(); p = v.presenter
        p.selectPostEdgeSpectra(self.spectraFileName)
        p.selectPostEdgeSpectra(None)
        self.assertEquals(None, v.postEdgeSpectrum)
        self.assertEquals(True, v.updatePlotCalled)

    def test_invalid_pre_edge_file_name(self):
        v = MockView(); p = v.presenter
        p.selectPreEdgeSpectra('invalid.filename')
        self.assertEquals(None, v.preEdgeSpectrum)
        self.assertEquals(True, v.updatePlotCalled)
        self.assertNotEqual(v.UNSET, v.errorMessage)

    def test_invalid_post_edge_file_name(self):
        v = MockView(); p = v.presenter
        p.selectPostEdgeSpectra('invalid.filename')
        self.assertEquals(None, v.postEdgeSpectrum)
        self.assertEquals(True, v.updatePlotCalled)
        self.assertNotEqual(v.UNSET, v.errorMessage)


if __name__ == '__main__':
    unittest.main()
