#!/usr/bin/env python
# Name: test_roiset.py
# Purpose: Unit tests for libsetuproi.roiset.RoiList
# Author: Ken McIvor <mcivor@iit.edu>
#
# Copyright 2006 Illinois Institute of Technology
#
# See the file "LICENSE" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.

import unittest
from libsetuproi.roilist import RoiList


class TestRoiList(unittest.TestCase):
    def test_get_roi(self):
        rl = RoiList()
        self.assertEquals((None, None), rl.getROI(0, 0))

    def test_set_roi(self):
        rl = RoiList()
        rl.setROI(0, 0, 100, 200)
        self.assertEquals((100, 200), rl.getROI(0, 0))

    def test_get_all_rois(self):
        rl = RoiList()
        self.assertEquals([(None, None)]*19, rl.getChannelROIs(0))

    def test_is_empty(self):
        rl = RoiList()
        self.assert_(rl.isEmpty(), 'rl should be empty')
        rl.setROI(0, 0, 100, 200)
        self.assert_(not rl.isEmpty(), 'rl should not be empty')
        rl.setROI(0, 0, None, None)
        self.assert_(rl.isEmpty(), 'rl should be empty')


class TestSetRoi(unittest.TestCase):
    def _testSetROI(self, start, end, resStart, resEnd):
        rl = RoiList()
        rl.setROI(0, 0, start, end)
        self.assertEquals((resStart, resEnd), rl.getROI(0, 0))

    def test_set_none(self):
        self._testSetROI(None, None, None, None)

    def test_set_start_none(self):
        self._testSetROI(None, 100, None, 100)

    def test_set_end_none(self):
        self._testSetROI(100, None, 100, None)

    def test_start_floor(self):
        self._testSetROI(100.1, 200, 100, 200)

    def test_end_ceil(self):
        self._testSetROI(100, 199.1, 100, 200)


class TestGetRoiIndexErrors(unittest.TestCase):
    def _testGetROI(self, i, j):
        rl = RoiList()
        self.assertRaises(ValueError, rl.getROI, i, j)
 
    def test_region_too_small(self):
        self._testGetROI(-1, 0)

    def test_region_too_large(self):
        self._testGetROI(100, 0)

    def test_channel_too_small(self):
        self._testGetROI(0, -1)

    def test_channel_too_large(self):
        self._testGetROI(0, 100)


class TestGetChannelRoisIndexErrors(unittest.TestCase):
    def _testGetChannelROIs(self, i):
        rl = RoiList()
        self.assertRaises(ValueError, rl.getChannelROIs, i)
 
    def test_region_too_small(self):
        self._testGetChannelROIs(-1)

    def test_region_too_large(self):
        self._testGetChannelROIs(100)


class TestSetRoiIndexErrors(unittest.TestCase):
    def _testSetROI(self, i, j):
        rl = RoiList()
        self.assertRaises(ValueError, rl.setROI, i, j, None, None)
 
    def test_region_too_small(self):
        self._testSetROI(-1, 0)

    def test_region_too_large(self):
        self._testSetROI(100, 0)

    def test_channel_too_small(self):
        self._testSetROI(0, -1)

    def test_channel_too_large(self):
        self._testSetROI(0, 100)


class TestToMotorScript(unittest.TestCase):
    def test_empty(self):
        rl = RoiList()
        self.assertEquals('', rl.toMotorScript())

    def test_start_none(self):
        rl = RoiList()
        rl.setROI(0, 0, None, 2)
        self.assertEquals('', rl.toMotorScript())

    def test_end_none(self):
        rl = RoiList()
        rl.setROI(0, 0, 1, None)
        self.assertEquals('', rl.toMotorScript())

    def test_contiguous(self):
        result = ('mca mcaxia0 set roi 0 1 2\n'
                  'mca mcaxia1 set roi 0 2 3\n'
                  'mca mcaxia2 set roi 0 3 4\n')

        rl = RoiList()
        rl.setROI(0, 0, 1, 2)
        rl.setROI(0, 1, 2, 3)
        rl.setROI(0, 2, 3, 4)
        self.assertEquals(result, rl.toMotorScript())

    def test_scattered(self):
        result = ('mca mcaxia0 set roi 0 1 2\n'
                  'mca mcaxia4 set roi 8 2 3\n'
                  'mca mcaxia18 set roi 15 3 4\n')

        rl = RoiList()
        rl.setROI(0,   0, 1, 2)
        rl.setROI(8,   4, 2, 3)
        rl.setROI(15, 18, 3, 4)
        self.assertEquals(result, rl.toMotorScript())


class TestFromMotorScript(unittest.TestCase):
    def test_empty(self):
        rl = RoiList.fromMotorScript('')
        self.assert_(rl.isEmpty(), 'rl should be empty')

    def test_contiguous(self):
        script = ('mca mcaxia0 set roi 0 1 2\n'
                  'mca mcaxia1 set roi 0 2 3\n'
                  'mca mcaxia2 set roi 0 3 4\n')

        rl = RoiList.fromMotorScript(script)
        self.assertEquals((1, 2), rl.getROI(0, 0))
        self.assertEquals((2, 3), rl.getROI(0, 1))
        self.assertEquals((3, 4), rl.getROI(0, 2))

    def test_scattered(self):
        script = ('mca mcaxia0 set roi 0 1 2\n'
                  'mca mcaxia4 set roi 8 2 3\n'
                  'mca mcaxia18 set roi 15 3 4\n')

        rl = RoiList.fromMotorScript(script)
        self.assertEquals((1, 2), rl.getROI(0, 0))
        self.assertEquals((2, 3), rl.getROI(8, 4))
        self.assertEquals((3, 4), rl.getROI(15, 18))


class TestFromMotorScriptErrors(unittest.TestCase):
    def test_empty(self):
        rl = RoiList.fromMotorScript('')
        self.assert_(rl.isEmpty(), 'rl should be empty')

    def test_comment(self):
        script = ('# mca mcaxia0 set roi 0 1 2\n'
                  'mca mcaxia1 set roi 0 2 3\n'
                  'mca mcaxia2 set roi 0 3 4\n')

        rl = RoiList.fromMotorScript(script)
        self.assertEquals((2, 3), rl.getROI(0, 1))
        self.assertEquals((3, 4), rl.getROI(0, 2))

    def test_bad_mca_name(self):
        script = ('mca mcaxia set roi 0 1 2\n'
                  'mca mca1 set roi 0 2 3\n'
                  'mca mcaxia2 set roi 0 3 4\n')

        rl = RoiList.fromMotorScript(script)
        self.assertEquals((3, 4), rl.getROI(0, 2))

    def test_blank_lines(self):
        script = ('\nmca mcaxia0 set roi 0 1 2\n'
                  '\nmca mcaxia1 set roi 0 2 3\n'
                  '\nmca mcaxia2 set roi 0 3 4\n')

        rl = RoiList.fromMotorScript(script)
        self.assertEquals((1, 2), rl.getROI(0, 0))
        self.assertEquals((2, 3), rl.getROI(0, 1))
        self.assertEquals((3, 4), rl.getROI(0, 2))

    def test_mangled_lines(self):
        script = ('mca mcaxia0 roi 0 1 2\n'
                  'mca mcaxia1 set 0 2 3\n'
                  'mca mcaxia2 set roi X Y 1\n')

        rl = RoiList.fromMotorScript(script)
        self.assert_(rl.isEmpty(), 'rl should be empty')



if __name__ == '__main__':
    unittest.main()
